package data

import (
	"context"
	"encoding/json"

	cwf "git.loafle.net/commons_go/websocket_fasthttp"
	oocmp "git.loafle.net/overflow/overflow_commons_go/modules/probe"
	probeM "git.loafle.net/overflow/overflow_commons_go/modules/probe/model"
	oogweg "git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
	"git.loafle.net/overflow/overflow_gateway_websocket/rpc"
	"github.com/valyala/fasthttp"
)

func newGatewayRPCHandler() rpc.GatewayRPCHandler {
	gh := &GatewayRPCHandlers{}

	return gh
}

type GatewayRPCHandlers struct {
	rpc.GatewayRPCHandlers
}

func (sh *GatewayRPCHandlers) Init(serverCTX cwf.ServerContext) error {
	return nil
}

func (sh *GatewayRPCHandlers) Handshake(socketCTX cwf.SocketContext, ctx *fasthttp.RequestCtx) (id string, extensionsHeader *fasthttp.ResponseHeader) {
	bMethod := ctx.Request.Header.Peek(oocmp.HTTPRequestHeaderKey_Probe_Method)
	if nil == bMethod {
		return "", nil
	}
	method := string(bMethod)

	switch method {
	case oocmp.HTTPRequestHeaderValue_Probe_Method_Connect:
	default:
		return "", nil
	}

	bProbeKey := ctx.Request.Header.Peek(oocmp.HTTPRequestHeaderKey_Probe_ProbeKey)
	if nil == bProbeKey {
		return "", nil
	}
	probeKey := string(bProbeKey)

	grpcCTX := context.Background()
	r, err := oogweg.Exec(grpcCTX, "ProbeService.readByProbeKey", []string{probeKey})
	if nil != err {
		return "", nil
	}
	probe := &probeM.Probe{}
	err = json.Unmarshal([]byte(r), probe)
	if nil != err {
		return "", nil
	}

	return probeKey, nil
}

func (sh *GatewayRPCHandlers) Validate() {
	sh.GatewayRPCHandlers.Validate()
}
